/***************************************************************************
                          kap_driverdetectpage.cpp  -  description
                             -------------------
    begin                : Tue Feb 8 2000
    copyright            : (C) 2000 by Michael Goffioul
    email                : goffioul@imec.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qlabel.h>
#include <qtextview.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <klocale.h>

#include "kap_driverdetectpage.h"
#include "kprinterdescription.h"
#include "kap_manumodeldico.h"

#define	DRIVER_AUTOMATIC	0
#define	DRIVER_MANUAL		1

Kap_DriverDetectPage::Kap_DriverDetectPage(QWidget *parent, const char *name )
:Kap_WizardPage(parent,name)
{
	setTitle("Driver selection method");

	group_ = new QButtonGroup(this);
	group_->setExclusive(true);
	connect(group_, SIGNAL(clicked(int)), SLOT(slotTypeChanged(int)));
        group_->hide();

        QRadioButton	*b1 = new QRadioButton(i18n("Automatic (recommanded)"), this);
	group_->insert(b1, DRIVER_AUTOMATIC);
        QRadioButton	*b2 = new QRadioButton(i18n("Manual"), this);
	group_->insert(b2, DRIVER_MANUAL);

	QLabel	*l1 = new QLabel(this);
	l1->setText(i18n("<p>One or more drivers have been detected for your printer. However you may want to select a driver manually. What kind of method for driver selection do you want to use ?</p>"));

	view_ = new QTextView(this);
	view_->setPaper(colorGroup().background());
	view_->setTextFormat(Qt::RichText);

	// layout management
	QVBoxLayout	*mainLayout_ = new QVBoxLayout(this, 10, 5);
        mainLayout_->addWidget(l1, 0);
        mainLayout_->addWidget(b1, 0);
        mainLayout_->addWidget(b2, 0);
        mainLayout_->addWidget(view_);

	group_->setButton(DRIVER_AUTOMATIC);
	slotTypeChanged(DRIVER_AUTOMATIC);
}

Kap_DriverDetectPage::~Kap_DriverDetectPage()
{
}

Kap_Main::e_PageType Kap_DriverDetectPage::getNextPage(QString&, bool fake)
{
	if (fake) return Kap_Main::ModelSelector;
	int	t = group_->id(group_->selected());
	KPrinterDescription	*descr = KPrinterDescription::get();
	descr->setPnp((t == DRIVER_AUTOMATIC));
        if (t == DRIVER_AUTOMATIC)
	{
		if (descr->getPrinterValues()) return Kap_Main::DriverConfig;
		else return Kap_Main::DriverSelector;
	}
        else return Kap_Main::ModelSelector;
}

Kap_Main::e_PageType Kap_DriverDetectPage::getType() const
{
	return Kap_Main::DriverDetectType;
}

void Kap_DriverDetectPage::slotTypeChanged(int type)
{
	if (type == DRIVER_AUTOMATIC)
		view_->setText(i18n("The drivers corresponding to your printer will be selected "
  				    "automatically. If several drivers are available, you will be "
                                    "asked to choose one of them."));
	else
		view_->setText(i18n("All drivers available in the KUPS database will be listed, "
                		    "and you will be asked to select one of them. You will also have "
                                    "the possibility to provide your own driver."));
}
