/***************************************************************************
                          kap_fileselector.cpp  -  description
                             -------------------
    begin                : Fri Jan 5 2001
    copyright            : (C) 2001 by Michael Goffioul
    email                : goffioul@imec.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kap_fileselector.h"

#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <klocale.h>
#include <qlayout.h>
#include <kfiledialog.h>

#include "kprinterdescription.h"

Kap_FileSelector::Kap_FileSelector(QWidget *parent, const char *name ) : Kap_WizardPage(parent,name)
{
	setTitle(i18n("File selector"));

	filepath_ = new QLineEdit(this);
	QPushButton	*browse_ = new QPushButton(i18n("Browse..."), this);
	browse_->setFixedSize(browse_->sizeHint());
	filepath_->setFixedHeight(filepath_->sizeHint().height());
	QLabel	*lab = new QLabel(i18n("Select the file for printing:"), this);
	lab->setFixedHeight(lab->sizeHint().height());

	QVBoxLayout	*main_ = new QVBoxLayout(this,10);
	QHBoxLayout	*sub_ = new QHBoxLayout(0,0,5);
	sub_->addWidget(filepath_, 1);
	sub_->addWidget(browse_, 0);
	main_->addStretch(1);
	main_->addWidget(lab, 0);
	main_->addLayout(sub_, 0);
	main_->addStretch(2);

	connect(browse_, SIGNAL(clicked()), SLOT(slotBrowse()));	
}

Kap_FileSelector::~Kap_FileSelector(){
}

Kap_Main::e_PageType Kap_FileSelector::getType() const
{
	return Kap_Main::FileSelector;
}

bool Kap_FileSelector::canNextPage(QString& msg)
{
	msg.truncate(0);
	if (filepath_->text().isEmpty())
		msg = i18n("You must specify a file !");
	else if (filepath_->text()[0] != '/')
		msg = i18n("Not an absolute path name !");
	else
	{
		QFileInfo	fi(filepath_->text());
		if (fi.exists() && fi.isDir())
			msg = i18n("\"%1\" is a directory !").arg(fi.absFilePath());
		else if (!fi.dir().exists())
			msg = i18n("Directory \"%1\" does not exist !").arg(fi.dir().absPath());
	}
	return msg.isEmpty();
}

Kap_Main::e_PageType Kap_FileSelector::getNextPage(QString&, bool)
{
	QString	uri = "file:" + filepath_->text();
	KPrinterDescription::get()->setURI(uri);
	return Kap_Main::ModelSelector;
}

void Kap_FileSelector::slotBrowse()
{
	QString	filename = KFileDialog::getSaveFileName(filepath_->text(), QString::null, this, i18n("File selection"));
	if (!filename.isEmpty())
		filepath_->setText(filename);
}
