#include <qstring.h>
#include <qstringlist.h>

#include <config.h>

#include "kap_ipputil.h"
#include "kprintermanager.h"
#include "kups_ipprequest.h"
#include "kups_attributesiterator.h"
#include "kups_infos.h"
#include <cups/http.h>
#include <string.h>

QString	Kap_IppUtil::server_;
QString	Kap_IppUtil::user_;
QString	Kap_IppUtil::passwd_;
int	Kap_IppUtil::port_;

void Kap_IppUtil::saveConfig()
{
	server_ = Kups_Infos::get()->getServer().data();
	user_ = Kups_Infos::get()->getUser().data();
	passwd_ = Kups_Infos::get()->getPassword().data();
	port_ = Kups_Infos::get()->getPort();
}

void Kap_IppUtil::restoreConfig()
{
	Kups_Infos::get()->setServer(server_);
	Kups_Infos::get()->setUser(user_);
	Kups_Infos::get()->setPassword(passwd_);
	Kups_Infos::get()->setPort(port_);
}

int Kap_IppUtil::getAvailableBackends()
{
#ifdef CUPS_VERSION_1_0
	KPrinterManager::setLastError("Backend listing : not supported in CUPS 1.0");
	return 0;
#endif

	Kups_IppRequest	req;
	QString	uri;

	uri = Kups_Infos::get()->getServerPortIppUri() + "/printers/";
#ifndef CUPS_VERSION_1_0
	req.setOperation(CUPS_GET_DEVICES);
#endif
	req.setUri("printer-uri",uri);
	if (!req.doRequest("/"))
	{
		return 0;
	}
	else
	{
		Kups_AttributesIterator	*iter = req.getAttributesIterator();
		ipp_attribute_t	*attr;
		int	flag(0);
		while ((attr=iter->next()) != 0)
		{
			if (strcmp(attr->name,"device-uri") == 0)
			{
				if (strncmp(attr->values[0].string.text,"socket",6) == 0) flag |= _SOCKET;
				else if (strncmp(attr->values[0].string.text,"parallel",8) == 0) flag |= _PARALLEL;
				else if (strncmp(attr->values[0].string.text,"serial",6) == 0) flag |= _SERIAL;
				else if (strncmp(attr->values[0].string.text,"smb",3) == 0) flag |= _SMB;
				else if (strncmp(attr->values[0].string.text,"lpd",3) == 0) flag |= _LPD;
				else if (strncmp(attr->values[0].string.text,"usb",3) == 0) flag |= _USB;
				else if (strncmp(attr->values[0].string.text,"file",4) == 0) flag |= _FILE;
				else if (strncmp(attr->values[0].string.text,"http",4) == 0) flag |= _HTTP;
				else if (strncmp(attr->values[0].string.text,"ipp",3) == 0) flag |= _IPP;
			}
		}
		return flag;
	}
}

bool Kap_IppUtil::hasCupsServer(const QString& host, int port)
{
	http_t	*HTTP = httpConnect(host.data(),port);
	if (HTTP)
	{
		httpClose(HTTP);
		return true;
	}
	else
	{
		return false;
	}
}

QStrList* Kap_IppUtil::getPrinterList(const QString& host, int port, const QString& user, const QString& passwd)
{
	saveConfig();

	Kups_Infos::get()->setServer(host);
	Kups_Infos::get()->setUser(user);
	Kups_Infos::get()->setPassword(passwd);
	Kups_Infos::get()->setPort(port);

	Kups_IppRequest	request;
	QString	uri = Kups_Infos::get()->getServerPortIppUri() + "/printers/";
	QStrList	*list(0);
	request.setOperation(CUPS_GET_PRINTERS);
	request.setUri("printer-uri",uri);
	if (request.doRequest("/printers/"))
	{
		list = new QStrList;
		list->setAutoDelete(true);
		Kups_AttributesIterator	*iter = request.getAttributesIterator();
		ipp_attribute_t	*attr;
		while ((attr=iter->next()) != 0)
		{
			if (strcmp(attr->name,"printer-name") == 0)
				list->append(attr->values[0].string.text);
		}
	}

	restoreConfig();
	return list;
}

QList<DeviceInfo>* Kap_IppUtil::getDevicePrinters(QStrList *backends)
{
#ifdef CUPS_VERSION_1_0
	KPrinterManager::setLastError("Backend listing : not supported in CUPS 1.0");
	return 0;
#endif

	Kups_IppRequest	request;
	QString	uri = Kups_Infos::get()->getServerPortIppUri() + "/printers/";
	
#ifndef CUPS_VERSION_1_0
	request.setOperation(CUPS_GET_DEVICES);
#endif
	request.setUri("printer-uri",uri);
	if (!request.doRequest("/")) return 0;
	else
	{
		QList<DeviceInfo>	*list = new QList<DeviceInfo>;
		DeviceInfo		*info(0);
		Kups_AttributesIterator	*iter = request.getAttributesIterator();
		ipp_attribute_t	*attr = iter->next();

		list->setAutoDelete(true);
		while (attr)
		{
			if (!info) info = new DeviceInfo;
			if (strcmp(attr->name,"device-class") == 0) info->Class = attr->values[0].string.text;
			else if (strcmp(attr->name,"device-info") == 0) info->Info = attr->values[0].string.text;
			else if (strcmp(attr->name,"device-make-and-model") == 0) info->MakeAndModel = attr->values[0].string.text;
			else if (strcmp(attr->name,"device-uri") == 0) info->URI = attr->values[0].string.text;

			attr = iter->next();
			if (!attr || !attr->name)
			{
				bool	ok(false);
				for (unsigned int i=0;i<backends->count() && !ok;i++)
				{
					unsigned int	len(strlen(backends->at(i)));
					if (strncmp(info->URI.data(),backends->at(i),len) == 0 && info->URI.length() > len)
					{
						info->Type = i;
						list->append(info);
						ok = true;
					}
				}
				if (!ok) delete info;
				info = 0;
				if (attr) attr = iter->next();
			}
		}

		if (list->count() <= 0)
		{
			delete list;
			return 0;
		}
#ifdef DEBUG
for (int i=0;i<list->count();i++)
debug("%s, %s, %s, %s",list->at(i)->Class.data(),list->at(i)->Info.data(),list->at(i)->MakeAndModel.data(),list->at(i)->URI.data());
#endif
		return list;
	}
}

QList<DeviceInfo>* Kap_IppUtil::getDevicePrinters(const char *backend)
{
	QStrList	list;
	list.setAutoDelete(true);
	list.append(backend);
	return getDevicePrinters(&list);
}

QList<DeviceInfo>* Kap_IppUtil::getLocalPrinters()
{
	QStrList	list;
	list.setAutoDelete(true);
	list.append("parallel");
	list.append("serial");
	list.append("usb");
	return getDevicePrinters(&list);
}

QList<DeviceInfo>* Kap_IppUtil::getParallelPrinters()
{
	return getDevicePrinters("parallel");
}

QList<DeviceInfo>* Kap_IppUtil::getSerialPrinters()
{
	return getDevicePrinters("serial");
}

QList<DeviceInfo>* Kap_IppUtil::getUSBPrinters()
{
	return getDevicePrinters("usb");
}
