#include "kap_printerconfig.h"

#include <klocale.h>
#include <qtcups/qppdwidget.h>
#include <qmessagebox.h>


Kap_PrinterConfig::Kap_PrinterConfig(global_ppd_file_t *ppdfile, QWidget *parent)
	: KDialogBase(parent,"printer-config",true,"",Ok|Cancel)
{
	useConflict_ = true;

	ppdwidget_ = new QPPDWidget(this);
	setMainWidget(ppdwidget_);
	internal_ppdfile_ = 0;
	ppdfile_ = 0;
	if (ppdfile)
		init(ppdfile);
	ppdwidget_->setAllowNonChangeable(true);
	setFixedSize( QSize(400,420) );
}

Kap_PrinterConfig::~Kap_PrinterConfig()
{
}

void Kap_PrinterConfig::init(global_ppd_file_t *ppdfile)
{
	if (ppdfile == 0) return;
	ppdfile_ = ppdfile;
	// make an internal copy of the PPD structure to work on
	// (the QPPDWidget automatically mark the selected options on the fly) so if the
	// user clicks Cancel, the options are already marked !!)
	QString	tmpfile = cupsTempFile(NULL, 0);
	writeGlobalPpdFile(ppdfile_, tmpfile.data(), ppdfile_->ppd_filename);
	internal_ppdfile_ = globalPpdOpenFile(tmpfile.data());
	unlink(tmpfile.data());
	if (internal_ppdfile_ == 0) return;
	globalPpdMarkDefaults(internal_ppdfile_);

	ppdwidget_->initialize(internal_ppdfile_);
}

void Kap_PrinterConfig::slotOk()
{
	if (ppdwidget_->hasConflict() && useConflict_)
		QMessageBox::critical(this,i18n("Printer configuration"),i18n("You must resolve conflicts before continuing !"),QMessageBox::Ok|QMessageBox::Default,0);
	else
	{
		if (ppdfile_) ppdwidget_->markChoices(ppdfile_);
		KDialogBase::accept();
	}
}

void Kap_PrinterConfig::setUseConflict(bool on)
{
	useConflict_ = on;
	//showButtonCancel(on);
}
