	/***************************************************************************
                          kbuffer.cpp  -  description
                             -------------------
    begin                : Tue Jan 25 2000
    copyright            : (C) 2000 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "kbuffer.h"

#include <qstring.h>
#include <qfile.h>



//-------------------------------------------------------------------------------
KBuffer::KBuffer()
:buffer_(),
pos_(0)
{
}
//-------------------------------------------------------------------------------
KBuffer::KBuffer(const QString& filename)
:pos_(0)
{
	load(filename);
}
//-------------------------------------------------------------------------------
KBuffer::KBuffer(const KBuffer& other)
:buffer_(other.buffer_),
pos_(0)
{
}
//-------------------------------------------------------------------------------
KBuffer::~KBuffer()
{
}

//-------------------------------------------------------------------------------
bool KBuffer::dump(const QString& fileName) const
{
	QFile file( fileName.data() );
	if (!file.open( IO_WriteOnly ))
	{
		return false;
	}

	bool ret = true;	
	if (file.writeBlock(buffer_) == -1)
	{
		ret = false;
	}
	
	file.close();
	return ret;
}
//-------------------------------------------------------------------------------
bool KBuffer::load(const QString& fileName)
{
	QFile file( fileName.data() );
	if (!file.open( IO_ReadOnly ))
	{
		return false;
	}
	
	char	*charbuf = new char[file.size()+1];
	charbuf[file.size()] = 0;
	bool	value(true);
	if (file.readBlock(charbuf, file.size()) != -1)
	{
		buffer_.setRawData(charbuf, file.size()+1);
	}
	else
	{
		delete [] charbuf;
		value = false;
	}
	file.close();
	return value;

/*	char buf[1025];
	int len = 1024;
	int readed = 0;
	while( (readed = file.readLine(buf, len)) >= 0 )
	{
		buf[readed] = 0;
		append(buf, readed);
	}

	file.close();
	
	return true;*/
}

//-------------------------------------------------------------------------------
bool KBuffer::append(const KBuffer& buf)
{
	return append(buf.data(), buf.size());
}
//-------------------------------------------------------------------------------
bool KBuffer::append(const QString& str)
{
	return append( str.data(), str.length() );
}
//-------------------------------------------------------------------------------
bool KBuffer::append(const char* buf, int len)
{
	int oldLen = size();
	const char* oldBuf = data();
	int allLen = len + oldLen;
	char* allBuf = new char[oldLen + len];
	
	memcpy(allBuf, oldBuf, oldLen);
	char* t = allBuf + oldLen;
	memcpy(t, buf, len);
	
	bool ret = assign(allBuf, allLen);
	delete [] allBuf;
	
	return ret;
}

//-------------------------------------------------------------------------------
bool KBuffer::assign(const KBuffer& buf)
{
	return assign(buf.data(), buf.size());
}
//-------------------------------------------------------------------------------
bool KBuffer::assign(const QString& str)
{
	return assign( str.data(), str.length() );
}
//-------------------------------------------------------------------------------
bool KBuffer::assign(const char* buf, int len)
{
	buffer_.resize(len+1);
	buffer_.duplicate(buf, len);
	return true;
}

//-------------------------------------------------------------------------------
const char* KBuffer::data() const
{
	return buffer_.data();
}

//-------------------------------------------------------------------------------
void KBuffer::reset()
{
	pos_ = 0;
}

//-------------------------------------------------------------------------------
const char* KBuffer::next(int atOneTime, int& bytes) const
{
	if (pos_ >= size())
	{
		bytes = 0;
		return 0;
	}
	
	int len = atOneTime;
	if (atOneTime + pos_ > size())
	{
		len = size() - pos_;
	}
	char* data = &buffer_.at(pos_);
	((KBuffer*)this)->pos_ += len;
	bytes = len;
	return data;
}

//-------------------------------------------------------------------------------
int KBuffer::size() const
{
	return buffer_.size();
}

//-------------------------------------------------------------------------------
bool KBuffer::replace(const QString& src, const QString& dest)
{
	QString	str(buffer_.data());
	int	pos = str.find(src);
	if (pos != -1)
	{
		str.replace(pos,src.length(),dest);
		assign(str);
		return true;
	}
	return false;
}
