#include "kpp_backendpage.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <klocale.h>

#include "kprinterdescription.h"
#include "kaddprinterwizard.h"

Kpp_BackendPage::Kpp_BackendPage(QWidget *parent, const char *name)
	: QWidget(parent,name)
{
	desc_ = 0;
	modified_ = false;
	initWidget();
}

Kpp_BackendPage::~Kpp_BackendPage()
{
}

void Kpp_BackendPage::initWidget()
{
	infoLabel_ = new QLabel(this);
	infoLabel_->setAlignment(Qt::AlignLeft|Qt::AlignTop|Qt::ExpandTabs);

	QPushButton	*change_ = new QPushButton(i18n("Change..."), this);
	change_->setFixedSize(change_->sizeHint());
	connect(change_,SIGNAL(clicked()),SLOT(changeClicked()));

	QVBoxLayout	*mainLayout_ = new QVBoxLayout(this, 10);
	mainLayout_->addWidget(infoLabel_, 1);
	mainLayout_->addWidget(change_, 0, Qt::AlignLeft|Qt::AlignVCenter);
}

void Kpp_BackendPage::changeClicked()
{
	Kap_AddPrinterWizard	wizard(this);
	wizard.setWizardControl(Kap_Main::BackendSelector, Kap_Main::ModelSelector, false);
	if (wizard.exec())
	{
		KPrinterDescription	*std = KPrinterDescription::get();
		desc_->setType(std->getType());
		desc_->setParportNumber(std->getParportNumber());
		desc_->setLoginInfo(std->getUser(), std->getPassword());
		desc_->setSMBInfo(std->getSMBWorkgroup(), std->getSMBServer(), std->getSMBPrinterName());
		desc_->setURI(std->getURI());
		desc_->setSocketInfo(std->getSocketPrinter(), std->getSocketPort());

		modified_ = true;

		setInfos(desc_);
	}
}

void Kpp_BackendPage::setInfos(KPrinterDescription *d)
{
	desc_ = d;

	QString	info_(i18n("Type: "));
	switch (d->getType())
	{
	   case KPrinterManager::e_USB:
	   case KPrinterManager::e_Serial:
	   case KPrinterManager::e_Parallel:
	   	info_.append(i18n("Local printer"));
	   	break;
	   case KPrinterManager::e_Socket:
	   	info_.append(i18n("Socket printer"));
	   	break;
	   case KPrinterManager::e_SMB:
	   	info_.append(i18n("SMB printer"));
	   	break;
	   case KPrinterManager::e_IPP:
	   	info_.append(i18n("IPP printer"));
	   	break;
	   case KPrinterManager::e_LPD:
	   	info_.append(i18n("LPD printer"));
	   	break;
	   case KPrinterManager::e_File:
	   	info_.append(i18n("File printer"));
	   	break;
		 default:
			break;
	}
	info_.append("\n");
	info_.append(i18n("URI: %1").arg(d->getDeviceUri()));

	infoLabel_->setText(info_);
	infoLabel_->setMinimumHeight(infoLabel_->sizeHint().height());
}

void Kpp_BackendPage::collectInfos(KPrinterDescription *d)
{
	d->setType(desc_->getType());
	d->setParportNumber(desc_->getParportNumber());
	d->setLoginInfo(desc_->getUser(), desc_->getPassword());
	d->setSMBInfo(desc_->getSMBWorkgroup(), desc_->getSMBServer(), desc_->getSMBPrinterName());
	d->setURI(desc_->getURI());
	d->setSocketInfo(desc_->getSocketPrinter(), desc_->getSocketPort());
}
