/***************************************************************************
                          kprinterdescription.cpp  -  description
                             -------------------
    begin                : Tue Feb 8 2000
    copyright            : (C) 2000 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kap_manumodeldico.h"
#include "kprinterdescription.h"
#include "kups_infos.h"
#include "kbuffer.h"

#include <klocale.h>


//----------------------------------------------------------------------------
KPrinterDescription* KPrinterDescription::unique_ = 0;


//----------------------------------------------------------------------------
KPrinterDescription::KPrinterDescription()
:printersList_(0),
printerValues_(0),
buffer_(0)
{
	reset();
}
KPrinterDescription::~KPrinterDescription()
{
	if (buffer_) delete buffer_;
	if (printersList_) delete printersList_;
}

void KPrinterDescription::reset()
{
	type_ = KPrinterManager::e_Invalid;
        parportNumber_ = -1;
        model_ = "";
        name_ = "";
        location_ = QString::null;
        ppd_ = "";
        description_ = QString::null;
        user_ = "";
        passwd_ = "";
        server_ = "";
        workgroup_ = "";
        printername_ = "";
	setPrintersList(0);
        uri_ = "";
        nickname_ = "";
        shortnickname_ = "";
        printerValues_ = 0;
	pnp_ = false;
        setBuffer(0);
	supportedbanners_.clear();
	supportedbanners_.append("none");
	startbanner_ = QString::null;
	endbanner_ = QString::null;
}

KPrinterDescription* KPrinterDescription::get()
{
	if (unique_ == 0)
	{
		unique_ = new KPrinterDescription();
	}
	return unique_;
}

void KPrinterDescription::release()
{
	if (unique_)
	{
		delete unique_;
		unique_ = 0;
	}
}

//------------------------------------------------------------------------
Kap_ModelItem* KPrinterDescription::getPrinterValues()
{
	return printerValues_;
}
//------------------------------------------------------------------------
void KPrinterDescription::setPrinterValues(Kap_ModelItem* values)
{
	printerValues_ = values;
}

//------------------------------------------------------------------------
QStrList* KPrinterDescription::getPrintersList()
{
	return printersList_;
}
void KPrinterDescription::setPrintersList(QStrList* list)
{
	if (printersList_) delete printersList_;
	printersList_ = list;
}

QString KPrinterDescription::getDescriptionString()
{
	QString	str;
	QString	temp;

	// first block : General infos
	str.append("<b>General</b><ul type=circle><li><u>Type</u>: ");
	switch (type_)
	{
		case KPrinterManager::e_Class:
			str.append("Printer class");
			break;
		case KPrinterManager::e_Parallel:
			str.append("Parallel printer");
			break;
		case KPrinterManager::e_Serial:
			str.append("Serial printer");
			break;
		case KPrinterManager::e_USB:
			str.append("USB printer");
			break;
		case KPrinterManager::e_IPP:
			str.append("IPP printer");
			break;
		case KPrinterManager::e_LPD:
			str.append("LPD printer");
			break;
		case KPrinterManager::e_SMB:
			str.append("SMB printer");
			break;
		case KPrinterManager::e_Socket:
			str.append("Network printer");
			break;
		case KPrinterManager::e_File:
			str.append("File printer");
			break;
		default:
			str.append("Unknown");
			break;
	}
	str.append("</li>");

	str.append( i18n("<li><u>Name</u>: %1</li>").arg(name_) );
	str.append( i18n("<li><u>Description</u>: %1</li>").arg(description_) );
	str.append( i18n("<li><u>Location</u>: %1</li>").arg(location_) );
	str.append( "</ul><br/>" );

	if (type_ != KPrinterManager::e_Class)
	{
		// second block : Backend infos
		str.append("<b>Backend</b><ul type=circle>");
		switch (type_)
		{
		   case KPrinterManager::e_Parallel:
		   case KPrinterManager::e_Serial:
		   case KPrinterManager::e_USB:
		   	if (!uri_.isEmpty()) temp = i18n("<li><u>URI</u>: %1</li>").arg(uri_);
		   	else temp = i18n("<li><u>Port</u>: %1</li>").arg(getParport());
		   	break;
		   case KPrinterManager::e_IPP:
		   case KPrinterManager::e_SMB:
		   	temp = i18n("<li><u>Host</u>: %1</li><li><u>Printer</u>: %2</li><li><u>Account</u>: %3</li>").arg(getSMBServer()).arg(getSMBPrinterName()).arg((getUser().isEmpty() ? QString("<guest>") : getUser()));
		   	break;
		   case KPrinterManager::e_LPD:
		   	temp = i18n("<li><u>Host</u>: %1</li><li><u>Queue</u>: %2</li>").arg(getSMBServer()).arg(getSMBPrinterName());
		   	break;
		   case KPrinterManager::e_Socket:
		   	temp = i18n("<li><u>Address</u>: %1</li><li><u>Port</u>: %2</li>").arg(getSocketPrinter()).arg(getSocketPort());
		   	break;
			 case KPrinterManager::e_File:
				temp = i18n("<li><u>File</u>: %1</li>").arg(uri_.right(uri_.length()-5));
				break;
		   default:
		   	break;
		}
		str.append( temp );
		str.append( "</ul><br/>" );

		// third block : driver infos
		str.append("<b>Driver</b><ul type=circle>");
		if (!printerValues_ && getPPD().isEmpty())
		{
			str.append( i18n("<li>None (raw printer)</li>") );
		}
		else
		{
			if (printerValues_ == 0)
			{
				str.append( i18n("<li><u>Type</u>: Provided driver</li><li><u>File</u>: %1</li>").arg(getPPD()) );
			}
			else
			{
				QString	ppd;
				ppd = printerValues_->getPpdFile();
				str.append( i18n("<li><u>Type</u>: Database driver</li><li><u>Driver</u>: %1</li>").arg((ppd.isEmpty() ? QString("<not available>") : ppd)) );
			}
			str.append( i18n("<li><u>Manufacturer</u>: %1</li><li><u>Model</u>: %2</li>").arg(manufacturer_).arg(model_) );
		}
		str.append( "</ul><br/>" );
	}
	else
	{ // class
		str.append(i18n("<b>Members</b><ul type=circle>"));
		if (printersList_)
			for (printersList_->first();printersList_->current();printersList_->next())
				str.append(QString("<li>%1</li>").arg(QString(printersList_->current())));
		str.append( "</ul>" );
	}
	
	return str;
}

//-----------------------------------------------------------------------------
const QString& KPrinterDescription::getManufacturer()
{
	return manufacturer_;
}
//-----------------------------------------------------------------------------
const QString& KPrinterDescription::getModel()
{
	return model_;
}

//-----------------------------------------------------------------------------
void KPrinterDescription::setManufacturer(const QString& str)
{
	manufacturer_ = str;
	cleanUp();
}
//-----------------------------------------------------------------------------
void KPrinterDescription::setModel(const QString& str)
{
	model_ = str;
	if (shortnickname_.isEmpty()) shortnickname_ = str;
        cleanUp();
}

//-----------------------------------------------------------------------------
KBuffer* KPrinterDescription::getBuffer()
{
	return buffer_;
}
void KPrinterDescription::setBuffer(KBuffer* b)
{
	// if we have a buffer, delete it (avoid memory leak).
	if (buffer_) delete buffer_;
	buffer_ = b;
}

const QString& KPrinterDescription::getDeviceUri()
{
	if (!uri_.isEmpty())
	{ // uri_ has been set, use this by default
		return uri_;
	}

	static QString uri;
	uri = "";	// initialize
	switch(type_)
	{
	case KPrinterManager::e_Parallel:
		uri.sprintf("parallel:%s", getParport().data());
		break;
	case KPrinterManager::e_Serial:
		ASSERT(0);
		break;
	case KPrinterManager::e_LPD:
		uri.sprintf("lpd://%s/%s",getSMBServer().data(),getSMBPrinterName().data());
		break;
	case KPrinterManager::e_IPP:
		uri.sprintf("ipp://%s:%d/printers/%s",getSocketPrinter().data(), getSocketPort(), getSMBPrinterName().data());
		break;
	case KPrinterManager::e_SMB:
		uri = "smb://";
		if (!getUser().isEmpty())
		{
			uri += getUser();
			if (!getPassword().isNull())
			{
			  uri += ":" + getPassword();
			}
			uri += "@";
		}

		uri += getSMBServer() + "/" + getSMBPrinterName();
		break;
	case KPrinterManager::e_Socket:
		uri.sprintf("socket://%s:%d",getSocketPrinter().data(),getSocketPort());
		break;
	case KPrinterManager::e_Class:
	default:
		ASSERT(0);
		break;
	}
	return uri;
}

void KPrinterDescription::setType(KPrinterManager::e_AddType type)
{
	type_ = type;
}
KPrinterManager::e_AddType KPrinterDescription::getType()
{
	return type_;
}

const QString& KPrinterDescription::getName()
{
	return name_;
}
const QString& KPrinterDescription::getPPD()
{
	return ppd_;
}
int KPrinterDescription::getParportNumber()
{
	return parportNumber_;
}
const QString& KPrinterDescription::getParport()
{
	static QString str;
	str.sprintf("/dev/lp%d", parportNumber_);
	return str;
}
const QString& KPrinterDescription::getLocation()
{
	return location_;
}
const QString& KPrinterDescription::getDescription()
{
	return description_;
}

const QString& KPrinterDescription::getUser()
{
	return user_;
}

const QString& KPrinterDescription::getPassword()
{
	return passwd_;
}

const QString& KPrinterDescription::getSMBWorkgroup()
{
	return workgroup_;
}

const QString& KPrinterDescription::getSMBServer()
{
	return server_;
}

const QString& KPrinterDescription::getSMBPrinterName()
{
	return printername_;
}

const QString& KPrinterDescription::getSocketPrinter()
{
	return server_;
}

int KPrinterDescription::getSocketPort()
{
	return parportNumber_;
}

const QString& KPrinterDescription::getURI()
{
	return uri_;
}

const QString& KPrinterDescription::getNickName()
{
	return nickname_;
}

const QString& KPrinterDescription::getShortNickName()
{
	return shortnickname_;
}

bool KPrinterDescription::isPnp()
{
	return pnp_;
}

const QString& KPrinterDescription::getStartBanner()
{
	return startbanner_;
}

const QString& KPrinterDescription::getEndBanner()
{
	return endbanner_;
}

const QStringList& KPrinterDescription::getSupportedBanners()
{
	return supportedbanners_;
}

//*****************************************************************************


void KPrinterDescription::setName(const QString& s)
{
	name_ = s;
}
void KPrinterDescription::setPPD(const QString& s)
{
	ppd_ = s;
}
void KPrinterDescription::setParportNumber(int pp)
{
	parportNumber_ = pp;
}
void KPrinterDescription::setLocation(const QString& s)
{
	location_ = s;
}
void KPrinterDescription::setDescription(const QString& s)
{
	description_ = s;
}

void KPrinterDescription::setLoginInfo(const QString& user, const QString& passwd)
{
	user_ = user;
	passwd_ = passwd;
}

void KPrinterDescription::setSMBInfo(const QString& work, const QString& server, const QString& printer)
{
	workgroup_ = work;
	server_ = server;
	printername_ = printer;
}

void KPrinterDescription::setLpdServer(const QString& host)
{
	server_ = host;
}

void KPrinterDescription::setLpdQueue(const QString& queue)
{
	printername_ = queue;
}

void KPrinterDescription::setSocketInfo(const QString& host, int port)
{
	server_ = host;
	parportNumber_ = port;
}

void KPrinterDescription::setURI(const QString& uri)
{
	uri_ = uri;
}

void KPrinterDescription::setNickName(const QString& name)
{
	nickname_ = name;
}

void KPrinterDescription::setShortNickName(const QString& name)
{
	shortnickname_ = name;
	cleanUp();
}

void KPrinterDescription::setPnp(bool on)
{
	pnp_ = on;
}

void KPrinterDescription::setStartBanner(const QString& banner)
{
	startbanner_ = banner;
}

void KPrinterDescription::setEndBanner(const QString& banner)
{
	endbanner_ = banner;
}

void KPrinterDescription::setSupportedBanners(const QStringList& banners)
{
	supportedbanners_ = banners;
}

void KPrinterDescription::cleanUp()
{
	if (!manufacturer_.isEmpty())
        {
		if (!model_.isEmpty()) model_ = cleanModel(manufacturer_, model_);
		if (!shortnickname_.isEmpty()) shortnickname_ = cleanModel(manufacturer_, shortnickname_);
        }
}
