/***************************************************************************
                          kups_jobsview.cpp  -  description
                             -------------------
    begin                : Fri Nov 26 1999
    copyright            : (C) 1999 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <kiconloader.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <klocale.h>
#include <qlist.h>

#include <cups/ipp.h>

#include "kups_jobsview.h"
#include "kups_jobitem.h"
#include "kprintermanager.h"
#include "kprinter.h"
#include "kups_printeritem.h"
#include "kups_jobsobserver.h"
#include "kjob.h"

extern void stopTimer();
extern void startTimer();


Kups_JobsView::Kups_JobsView(QWidget *parent, const char *name )
:QListView(parent,name),
selectedPrinter_(0)
//view_(0)
{
	createWidgets();
	initWidgets();
}
Kups_JobsView::~Kups_JobsView()
{
}

bool Kups_JobsView::createWidgets()
{
	setSelectionMode(QListView::Extended);
	setRootIsDecorated(false);
	setUpdatesEnabled(true);
	//setTreeStepSize(12);

	//columns
	addColumn(i18n("Name"));
	addColumn(i18n("ID"));
	addColumn(i18n("State"));
	addColumn(i18n("Printer"));
	addColumn(i18n("Owner"));
	addColumn(i18n("Size"));
	setSorting(1,true);
		
	observer_ = new Kups_JobsObserver(this);
	KPrinterManager::get()->addObserver(observer_);
	
	connect(	this, SIGNAL(rightButtonClicked(QListViewItem*, const QPoint&, int)),
						this, SLOT(slotRightOnItem(QListViewItem*, const QPoint&, int)) );
	
	return true;
}

#define	ID_POP_REMOVESEL	0
#define	ID_POP_REMOVEALL	1
#define	ID_POP_REFRESH		2
#define	ID_POP_HOLD		3
#define	ID_POP_RELEASE		4
#define	ID_POP_RESTART		5
#define	ID_POP_MOVE		100
void Kups_JobsView::slotRightOnItem(QListViewItem* it, const QPoint& point, int i)
{
	if (!it || i != 0) return;

	::stopTimer();

	// get jobs selected
	QList<KJob>	jobList, allJobs;
	jobList.setAutoDelete(false);
	allJobs.setAutoDelete(false);
	Kups_JobItem	*item = (Kups_JobItem*)firstChild();
	while (item)
	{
		// add only items with a valid Job
		if (item->getJob())
		{
			if (isSelected(item)) jobList.append(item->getJob());
			allJobs.append(item->getJob());
		}
		item = (Kups_JobItem*)item->nextSibling();
	}

	QPopupMenu	menu, menu2;
	if (jobList.count() == 0) return;	// do nothing
	else if (jobList.count() == 1)
	{ // only 1 job selected
		switch (jobList.first()->getState())
		{
		   case IPP_JOB_PROCESSING:
		   case IPP_JOB_STOPPED:
		   	break;
		   case IPP_JOB_HELD:
		   	menu.insertItem( SmallIcon("kups_enable"), i18n("Resume job"), ID_POP_RELEASE);
		   	break;
		   case IPP_JOB_PENDING:
		   	menu.insertItem( SmallIcon("stop"), i18n("Hold job"), ID_POP_HOLD);
		   	break;
		}
		menu.insertItem( SmallIcon("trashcan_empty"), i18n("Remove job"), ID_POP_REMOVESEL);
		int	nbPrinters = KPrinterManager::get()->getNbPrinters();
		for (int i=0;i<nbPrinters;i++)
		{
			KPrinter	*pr = KPrinterManager::get()->getPrinterAt(i);
			if (!pr || jobList.first()->getPrinter() == pr) continue;
			menu2.insertItem( SmallIcon("kups_printer"), pr->getName(), ID_POP_MOVE+i+1);
		}
		menu.insertItem( SmallIcon("kups_printer"), i18n("Move to printer..."), &menu2, ID_POP_MOVE);
	}
	else
		menu.insertItem( SmallIcon("trashcan_empty"), i18n("Remove selected jobs"), ID_POP_REMOVESEL);

	menu.insertSeparator();
	menu.insertItem( SmallIcon("trashcan_empty"), i18n("Remove all jobs"), ID_POP_REMOVEALL);
	menu.insertSeparator();
	menu.insertItem( SmallIcon("reload"), i18n("Refresh"), ID_POP_REFRESH);

	int ret = menu.exec(point);

	bool retVal = true;
	switch (ret)
	{
	   case ID_POP_REMOVESEL:
	   	for (jobList.first();jobList.current() && retVal;jobList.next())
	   		retVal = jobList.current()->sendCommand(KJob::Remove);
	   	break;
	   case ID_POP_REMOVEALL:
	   	for (allJobs.first();allJobs.current() && retVal;allJobs.next())
	   		retVal = allJobs.current()->sendCommand(KJob::Remove);
	   	break;
	   case ID_POP_REFRESH:
	   	retVal = true;
	   	break;
	   case ID_POP_HOLD:
	   	retVal = jobList.first()->sendCommand(KJob::Hold);
	   	break;
	   case ID_POP_RELEASE:
	   	retVal = jobList.first()->sendCommand(KJob::Release);
	   	break;
	   default:
	   	if (ret > 100 && ret < 200)
	   	{
	   		int	pIndex = ret - ID_POP_MOVE - 1;
	   		KPrinter	*pr = KPrinterManager::get()->getPrinterAt(pIndex);
	   		if (pr) retVal = jobList.first()->sendCommand(KJob::Move,pr);
	   	}
	   	else retVal = true;
	   	break;
	}

	if (retVal == false)
	{
		QString str;
		const char* error = (const char*)KPrinterManager::getLastError();
		str.sprintf("ERROR : %s", error);
		QMessageBox::critical(this,i18n("Error"),str,QMessageBox::Ok|QMessageBox::Default,0);
		KPrinterManager::setPassword("");
	}
	else
	{
		KPrinterManager::get()->refresh();
	}

	::startTimer();
	return;
}

void Kups_JobsView::setAllItemDiscarded(bool disc)
{
	Kups_JobItem* item = (Kups_JobItem*)firstChild();
	while(item != 0)
	{
		item->setIsDiscarded(disc);
		item = (Kups_JobItem*)item->nextSibling();
	}
}

void Kups_JobsView::removeAllDiscarded()
{
	Kups_JobItem* item = (Kups_JobItem*)firstChild();
	while(item != 0)
	{
		Kups_JobItem* temp = item;
		item = (Kups_JobItem*)item->nextSibling();
		if (temp->isDiscarded())
		{
			delete temp;
		}
	}
}

bool Kups_JobsView::initWidgets()
{
	// discard
	setAllItemDiscarded(true);
	
	if (selectedPrinter_ != 0 && !selectedPrinter_->isDiscarded())
	{
		Kups_JobItem* item;
		for (int i=0;i<selectedPrinter_->getNbJobs();i++)
		{
			KJob* job = selectedPrinter_->getJobAt(i);
			if (job->isDiscarded()) continue;
			item = getItemByJobID( job->getID() );
			if (item == 0)
			{
				item = new Kups_JobItem(this);
			}
			item->setJob(job);
			item->setIsDiscarded(false);
			item->setOpen(true);
			switch (job->getState())
			{
			   case IPP_JOB_PROCESSING:
			   	item->setPixmap(0, SmallIcon("kups_job_process"));
			   	break;
			   case IPP_JOB_HELD:
			   	item->setPixmap(0, SmallIcon("kups_job_stopped"));
			   	break;
			   case IPP_JOB_STOPPED:
			   	item->setPixmap(0, SmallIcon("kups_job_error"));
			   	break;
			   default:
			   	item->setPixmap(0, SmallIcon("kups_job"));
			   	break;
			}
		}
	}
	
	//delete
	removeAllDiscarded();
	
	return true;
}

Kups_JobItem* Kups_JobsView::getItemByJobID(int id)
{
	Kups_JobItem* item = (Kups_JobItem*)firstChild();
	while(item != 0)
	{
		if (id == item->getJob()->getID())
		{
			return item;
		}
		item = (Kups_JobItem*)item->nextSibling();
	}
	return 0;
}

void Kups_JobsView::selectionChanged(QListViewItem* item)
{
	if (!item || item->depth() < 2) selectedPrinter_ = 0;
	else
	{
		Kups_PrinterItem* pi = (Kups_PrinterItem*)item;
		selectedPrinter_ = pi->getPrinter();
	}
	initWidgets();
}
