#include "kups_printerpropview.h"

#include <qgroupbox.h>
#include <klocale.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <qtextview.h>

#include <cups/cups.h>

#include "kprinter.h"
#include "kaddprinterwizard.h"
#include "kups_printeritem.h"
#include "kprintermanager.h"
#include "kbuffer.h"

extern void stopTimer();
extern void startTimer();

KPrinterManager::e_AddType typeFromURI(const QString& uri)
{
	int	p(-1);
	QString	prot;
	if ((p=uri.find(':')) != -1)
	{
		prot = uri.left(p);
		if (prot == "file") return KPrinterManager::e_File;
		else if (prot == "parallel") return KPrinterManager::e_Parallel;
		else if (prot == "serial") return KPrinterManager::e_Serial;
		else if (prot == "ipp" || prot == "http") return KPrinterManager::e_IPP;
		else if (prot == "lpd") return KPrinterManager::e_LPD;
		else if (prot == "smb") return KPrinterManager::e_SMB;
		else if (prot == "socket") return KPrinterManager::e_Socket;
		else if (prot == "usb") return KPrinterManager::e_USB;
		else return KPrinterManager::e_Invalid;
	}
	return KPrinterManager::e_Invalid;
}

ToolLabel::ToolLabel(QWidget *parent, const char *name) : QLabel(parent, name)
{
	setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
}

ToolLabel::~ToolLabel()
{
}

void ToolLabel::checkTip()
{
	QToolTip::remove(this);
	if (fontMetrics().width(text()) > width())
		QToolTip::add(this, text());
}

void ToolLabel::resizeEvent(QResizeEvent *e)
{
	QLabel::resizeEvent(e);
        checkTip();
}

void ToolLabel::setText(const QString& txt)
{
	QLabel::setText(txt);
	checkTip();
}

//----------------------------------------------------------------------------------------

Kups_PrinterPropView::Kups_PrinterPropView(QWidget *parent, const char *name)
	: QWidget(parent, name)
{
        membersList_.setAutoDelete(true);
        createWidget();
}

Kups_PrinterPropView::~Kups_PrinterPropView()
{
}

void Kups_PrinterPropView::createWidget()
{
	genbox_ = new QGroupBox(i18n("General"), this);
	backendbox_ = new QGroupBox(i18n("Backend"), this);
	driverbox_ = new QGroupBox(i18n("Driver"), this);
	membersbox_ = new QGroupBox(i18n("Members"), this);
	bannerbox_ = new QGroupBox(i18n("Banners"), this);

	QLabel	*l1 = new QLabel(i18n("Name:"), genbox_);
	QLabel	*l2 = new QLabel(i18n("Description:"), genbox_);
	QLabel	*l3 = new QLabel(i18n("Location:"), genbox_);
	QLabel	*l4 = new QLabel(i18n("Type:"), backendbox_);
	QLabel	*l5 = new QLabel(i18n("URI:"), backendbox_);
	QLabel	*l6 = new QLabel(i18n("Manufacturer:"), driverbox_);
	QLabel	*l7 = new QLabel(i18n("Model name:"), driverbox_);
	QLabel	*l8 = new QLabel(i18n("Driver infos:"), driverbox_);
	QLabel	*l9 = new QLabel(i18n("Starting banner:"), bannerbox_);
	QLabel	*l10 = new QLabel(i18n("Ending banner:"), bannerbox_);

	nameLabel_ = new ToolLabel(genbox_);
	descLabel_ = new ToolLabel(genbox_);
	locLabel_ = new ToolLabel(genbox_);
	typeLabel_ = new ToolLabel(backendbox_);
	uriLabel_ = new ToolLabel(backendbox_);
	manuLabel_ = new ToolLabel(driverbox_);
	modelLabel_ = new ToolLabel(driverbox_);
	nickLabel_ = new QLabel(driverbox_);
	nickLabel_->setTextFormat(Qt::RichText);
	nickLabel_->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
/*	membersLabel_ = new QLabel(membersbox_);
	membersLabel_->setTextFormat(Qt::RichText);
	membersLabel_->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));*/
	membersView_ = new QTextView(membersbox_);
	membersView_->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
	membersView_->setPaper(colorGroup().background());
	membersView_->setFrameStyle(QFrame::NoFrame);
	startLabel_ = new QLabel(bannerbox_);
	endLabel_ = new QLabel(bannerbox_);

	genchange_ = new QPushButton(i18n("Change"), genbox_);
	backendchange_ = new QPushButton(i18n("Change"), backendbox_);
	driverchange_ = new QPushButton(i18n("Change"), driverbox_);
	memberschange_ = new QPushButton(i18n("Change"), membersbox_);
	bannerchange_ = new QPushButton(i18n("Change"), bannerbox_);
        connect(genchange_, SIGNAL(clicked()), SLOT(slotGeneralChange()));
        connect(backendchange_, SIGNAL(clicked()), SLOT(slotBackendChange()));
        connect(driverchange_, SIGNAL(clicked()), SLOT(slotDriverChange()));
        connect(memberschange_, SIGNAL(clicked()), SLOT(slotMembersChange()));
        connect(bannerchange_, SIGNAL(clicked()), SLOT(slotBannerChange()));

	QVBoxLayout	*mainLayout_ = new QVBoxLayout(this, 10, 10);
	QHBoxLayout	*genLayout_ = new QHBoxLayout(genbox_, 10, 5);
	QHBoxLayout	*backendLayout_ = new QHBoxLayout(backendbox_, 10, 5);
	QHBoxLayout	*driverLayout_ = new QHBoxLayout(driverbox_, 10, 5);
	QHBoxLayout	*membersLayout_ = new QHBoxLayout(membersbox_, 10, 5);
	QHBoxLayout	*bannerLayout_ = new QHBoxLayout(bannerbox_, 10, 5);
	QVBoxLayout	*g1 = new QVBoxLayout(0, 0, 3), *g2 = new QVBoxLayout(0, 0, 3), *g3 = new QVBoxLayout(0, 0, 0);
	QVBoxLayout	*b1 = new QVBoxLayout(0, 0, 3), *b2 = new QVBoxLayout(0, 0, 3), *b3 = new QVBoxLayout(0, 0, 0);
	QVBoxLayout	*d1 = new QVBoxLayout(0, 0, 3), *d2 = new QVBoxLayout(0, 0, 3), *d3 = new QVBoxLayout(0, 0, 0);
	QVBoxLayout	*m1 = new QVBoxLayout(0, 0, 3), *m2 = new QVBoxLayout(0, 0, 0);
	QVBoxLayout	*bn1 = new QVBoxLayout(0, 0, 3), *bn2 = new QVBoxLayout(0, 0, 3), *bn3 = new QVBoxLayout(0, 0, 0);

	// General layout
	genLayout_->addLayout(g1, 0);
	genLayout_->addLayout(g2, 1);
	genLayout_->addLayout(g3, 0);
	g1->addSpacing(10);
	g1->addWidget(l1, 0);
	g1->addWidget(l2, 0);
	g1->addWidget(l3, 0);
	g2->addSpacing(10);
	g2->addWidget(nameLabel_, 0);
	g2->addWidget(descLabel_, 0);
	g2->addWidget(locLabel_, 0);
	g3->addStretch(1);
	g3->addWidget(genchange_, 0);

	// Backend layout
	backendLayout_->addLayout(b1, 0);
	backendLayout_->addLayout(b2, 1);
	backendLayout_->addLayout(b3, 0);
	b1->addSpacing(10);
	b1->addWidget(l4, 0);
	b1->addWidget(l5, 0);
	b2->addSpacing(10);
	b2->addWidget(typeLabel_, 0);
	b2->addWidget(uriLabel_, 0);
	b3->addStretch(1);
	b3->addWidget(backendchange_, 0);


	// Driver layout
	driverLayout_->addLayout(d1, 0);
	driverLayout_->addLayout(d2, 1);
	driverLayout_->addLayout(d3, 0);
	d1->addSpacing(10);
	d1->addWidget(l6, 0);
	d1->addWidget(l7, 0);
	d1->addWidget(l8, 0);
	d1->addStretch(1);
	d2->addSpacing(10);
	d2->addWidget(manuLabel_, 0);
	d2->addWidget(modelLabel_, 0);
	d2->addWidget(nickLabel_, 0);
	d2->addStretch(1);
	d3->addStretch(1);
	d3->addWidget(driverchange_, 0);

	// Members layout
	membersLayout_->addLayout(m1, 1);
	membersLayout_->addLayout(m2, 0);
	m1->addSpacing(10);
	m1->addWidget(membersView_, 1);
	m2->addSpacing(10);
	m2->addStretch(1);
	m2->addWidget(memberschange_, 0);

	// Banner layout
	bannerLayout_->addLayout(bn1, 0);
	bannerLayout_->addLayout(bn2, 1);
	bannerLayout_->addLayout(bn3, 0);
	bn1->addSpacing(10);
	bn1->addWidget(l9, 0);
	bn1->addWidget(l10, 0);
	bn2->addSpacing(10);
	bn2->addWidget(startLabel_, 0);
	bn2->addWidget(endLabel_, 0);
	bn3->addStretch(1);
	bn3->addWidget(bannerchange_, 0);

        // Main layout
	mainLayout_->addWidget(genbox_, 0);
	mainLayout_->addWidget(backendbox_, 0);
	mainLayout_->addWidget(driverbox_, 0);
	mainLayout_->addWidget(membersbox_, 0);
	mainLayout_->addWidget(bannerbox_, 0);
	mainLayout_->addStretch(1);

	// Initially, assume a printer
	membersbox_->hide();
	bannerbox_->hide();
	setPrinter(0);
}

void Kups_PrinterPropView::slotGeneralChange()
{
	::stopTimer();

        KPrinterDescription	*std = KPrinterDescription::get();
        std->reset();
        std->setName(printer_->getName());
        std->setDescription(printer_->getDescription());
        std->setLocation(printer_->getLocation());
	if (printer_->isClass()) std->setType(KPrinterManager::e_Class);

	Kap_AddPrinterWizard	dlg(this);
        dlg.setWizardControl(Kap_Main::NameSelector, Kap_Main::NameSelector, true);
	if (dlg.exec())
        {
		KPrinterDescription	d;
                if (printer_->getName() != std->getName()) printer_->fillDescription(&d);
                d.setName(std->getName());
                d.setDescription(std->getDescription());
                d.setLocation(std->getLocation());
		if (printer_->isClass()) d.setType(KPrinterManager::e_Class);
                updatePrinter(&d);
        }

	::startTimer();
}

void Kups_PrinterPropView::slotBackendChange()
{
	::stopTimer();

	KPrinterDescription	*std = KPrinterDescription::get();
	std->reset();

	Kap_AddPrinterWizard	dlg(this);
	dlg.setWizardControl(Kap_Main::BackendSelector, Kap_Main::ModelSelector, false);
	if (dlg.exec())
	{
                KPrinterDescription	d;
                d.setName(printer_->getName());	// to avoid removal of printer (we need a printer name)
                d.setURI(std->getDeviceUri());
		updatePrinter(&d);
	}

	::startTimer();
}

void Kups_PrinterPropView::slotDriverChange()
{
	::stopTimer();

	KPrinterDescription	*std = KPrinterDescription::get();
        std->reset();
        std->setManufacturer(manuLabel_->text());
        std->setModel(modelLabel_->text());
        std->setShortNickName(modelLabel_->text());
	std->setURI(printer_->getLocalURI());
	std->setType(typeFromURI(std->getURI()));

	Kap_AddPrinterWizard	dlg(this);
        dlg.setWizardControl(Kap_Main::ModelSelector, Kap_Main::DriverConfig, true);
	if (dlg.exec())
        {
                KPrinterDescription	d;
                d.setName(printer_->getName());
                d.setPPD(std->getPPD());
                if (std->getBuffer())
                	d.setBuffer(new KBuffer(*(std->getBuffer())));
                updatePrinter(&d);
        }

	::startTimer();
}

void Kups_PrinterPropView::slotMembersChange()
{
	::stopTimer();

	KPrinterDescription	*std = KPrinterDescription::get();
        std->reset();
        if (membersList_.count() > 0)
        {
                QStrList	*newlist = new QStrList(membersList_);
                std->setPrintersList(newlist);
        }

	Kap_AddPrinterWizard	dlg(this);
        dlg.setWizardControl(Kap_Main::ClassSelector, Kap_Main::ClassSelector, true);
        if (dlg.exec())
        {
        	QStrList	*prs = std->getPrintersList();
                if (prs)
                {
                        KPrinterDescription	d;
                        d.setName(printer_->getName());
                        d.setType(KPrinterManager::e_Class);
                        d.setPrintersList(new QStrList(*prs));
                        updatePrinter(&d);
                }
        }

	::startTimer();
}

void Kups_PrinterPropView::slotBannerChange()
{
	::stopTimer();

	KPrinterDescription	*std = KPrinterDescription::get();
	std->reset();
	std->setSupportedBanners(printer_->getSupportedBanners());
	std->setStartBanner(printer_->getStartBanner());
	std->setEndBanner(printer_->getEndBanner());

	Kap_AddPrinterWizard	dlg(this);
	dlg.setWizardControl(Kap_Main::BannerSelector, Kap_Main::NameSelector, false);
	if (dlg.exec())
	{
                KPrinterDescription	d;
                d.setName(printer_->getName());	// to avoid removal of printer (we need a printer name)
		d.setStartBanner(std->getStartBanner());
		d.setEndBanner(std->getEndBanner());
		updatePrinter(&d);
	}

	::startTimer();
}

void Kups_PrinterPropView::setPrinter(KPrinter *printer)
{
        printer_ = printer;
        membersList_.clear();
	// allow modifications only for non-implicit local printers
	bool	canChange = (printer_ && !printer_->isImplicit() && !printer_->isRemote());
        if (!printer_)
        {
        	setupForPrinter(true);
                nameLabel_->setText("");
                descLabel_->setText("");
                locLabel_->setText("");
                manuLabel_->setText("");
                modelLabel_->setText("");
                nickLabel_->setText("");
                typeLabel_->setText("");
                uriLabel_->setText("");
                membersView_->setText("");
                startLabel_->setText("");
                endLabel_->setText("");
        }
        else
        {
        	bool	isClass = (printer_->isClass() || printer_->isImplicit());
                setupForPrinter(!isClass);
                KPrinterDescription	desc_;
                printer_->fillDescription(&desc_);
                nameLabel_->setText(printer_->getName());
                descLabel_->setText(printer_->getDescription());
                locLabel_->setText(printer_->getLocation());
                if (isClass)
                {
			QStrList	*prs = desc_.getPrintersList();
                        if (prs && prs->count() > 0)
                        {
                        	QString	str("<ul type=circle>");
                        	for (prs->first();prs->current();prs->next())
                                {
                                	str.append("<li>");
                                        str.append(prs->current());
                                        str.append("</li>");
                                        membersList_.append(prs->current());
                                }
                                str.append("</ul>");
                                membersView_->setText(str);
                        }
                        else
			{
				if (printer_->isImplicit()) membersView_->setText("<p>Members of implicit classes cannot be listed.</p>");
				else if (printer_->isRemote()) membersView_->setText(i18n("<p>Members of remote classes cannot be listed.</p>"));
				else membersView_->setText(i18n("<p>No printers found in this class.</p>"));
			}
                }
                else
                {
	                uriLabel_->setText(printer_->getLocalURI());
			QString	str;
			switch (desc_.getType())
                        {
                           case KPrinterManager::e_USB: str = i18n("USB printer"); break;
                           case KPrinterManager::e_Serial: str = i18n("Serial printer"); break;
                           case KPrinterManager::e_Parallel: str = i18n("Parallel printer"); break;
                           case KPrinterManager::e_Socket: str = i18n("Socket printer"); break;
                           case KPrinterManager::e_SMB: str = i18n("SMB printer"); break;
                           case KPrinterManager::e_IPP: str = i18n("IPP printer"); break;
                           case KPrinterManager::e_LPD: str = i18n("LPD printer"); break;
                           case KPrinterManager::e_File: str = i18n("File printer"); break;
                           default: str = i18n("Unknown"); break;
                        }
                        typeLabel_->setText(str);
                        if (!desc_.getModel().isEmpty())
                        {
                        	manuLabel_->setText(desc_.getManufacturer());
	                	modelLabel_->setText(desc_.getShortNickName());
				nickLabel_->setText(desc_.getNickName());
                        }
                        else
                        {
                        	manuLabel_->setText("");
                                modelLabel_->setText("");
                                nickLabel_->setText("No driver found");
                        }
			if (!printer_->isRemote())
			{
				startLabel_->setText(printer_->getStartBanner());
				endLabel_->setText(printer_->getEndBanner());
			}
		}
        }
	genchange_->setEnabled(canChange);
	backendchange_->setEnabled(canChange);
	driverchange_->setEnabled(canChange);
	memberschange_->setEnabled(canChange);
	if (canChange && !printer_->isClass())
		bannerbox_->show();
	else
		bannerbox_->hide();
}

void Kups_PrinterPropView::slotPrinterSelected(QListViewItem *item)
{
	if (!item || item->depth() < 2)
        {
                setPrinter(0);
        }
        else
        {
		Kups_PrinterItem	*item_ = (Kups_PrinterItem*)item;
		setPrinter(item_->getPrinter());
        }
}

void Kups_PrinterPropView::setupForPrinter(bool on)
{
	if (on)
        {
        	membersbox_->hide();
        	backendbox_->show();
		driverbox_->show();
        }
        else
        {
        	backendbox_->hide();
		driverbox_->hide();
        	membersbox_->show();
	}
}

void Kups_PrinterPropView::updatePrinter(KPrinterDescription *d)
{
        KPrinterManager	*mgr = KPrinterManager::get();
        if (!mgr->modifyPrinter(printer_, d))
        {
		QMessageBox::critical(this, i18n("Error"), i18n("Unable to modify printer !\n%1").arg(mgr->getLastError()), QMessageBox::Ok|QMessageBox::Default, 0);
        }
}
