/***************************************************************************
                          kups_printerpage.cpp  -  description
                             -------------------
    begin                : Mon Nov 22 1999
    copyright            : (C) 1999 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


// QT includes
#include <qpopupmenu.h>

// KDE includes
#include <qmessagebox.h>
#include <kiconloader.h>
#include <klocale.h>

// KUPS includes
#include "kups.h"
#include "kupsview.h"
#include "kups_printerview.h"

#include "kups_printeritem.h"
#include "kprintermanager.h"
#include "kprinter.h"
#include "kups_printerobserver.h"
#include "kups_testpagegenerator.h"
#include "kaddprinterwizard.h"
#include "kups_printerconfigurator.h"
#include "kups_infos.h"
#include "kinstancemanager.h"



Kups_PrinterView::Kups_PrinterView(QWidget *parent, const char *name, KupsView* view, bool allColumns)
:QListView(parent, name),
allColumns_(allColumns),
view_(view)
{
	createWidgets();
	initWidgets();
}
Kups_PrinterView::~Kups_PrinterView()
{
}


bool Kups_PrinterView::createWidgets()
{
	setMultiSelection(false);
	setRootIsDecorated(false);
	setTreeStepSize(12);
	setUpdatesEnabled(true);
	
	int nbCol = 1;
	if (allColumns_)
	{
		nbCol = KPrinter::getNbColumns();
	}
	for (int i=0;i<nbCol;i++)
	{
		QString col = KPrinter::getStaticColumnNameAt(i);	
		addColumn(col);
	}
	
	clear();
	all_ = new Kups_PrinterItem(this, allColumns_);
	all_->setText(0, i18n("CUPS on %1:%2").arg(Kups_Infos::get()->getServer()).arg(Kups_Infos::get()->getPort()));
	all_->setOpen(true);
	all_->setPrinterType(Kups_PrinterItem::ALL);
	all_->setPixmap(0, SmallIcon("kups_printer"));
	allPrinters_ = new Kups_PrinterItem(all_, allColumns_);
	allPrinters_->setText(0, i18n("Printers") );
	allPrinters_->setOpen(true);
	allPrinters_->setPixmap(0, SmallIcon("package"));
	allPrinters_->setPrinterType(Kups_PrinterItem::ALL_PRINTERS);
	allClasses_ = new Kups_PrinterItem(all_, allColumns_);
	allClasses_->setText(0, i18n("Classes") );
	allClasses_->setPixmap(0, SmallIcon("package"));
	allClasses_->setOpen(true);
	allClasses_->setPrinterType(Kups_PrinterItem::ALL_CLASSES);

	observer_ = new Kups_PrinterObserver(this);
	KPrinterManager::get()->addObserver(observer_);

	connect(	this, SIGNAL(rightButtonClicked(QListViewItem*, const QPoint&, int)),
						this, SLOT(slotRightOnItem(QListViewItem*, const QPoint&, int)) );

	return false;
}

bool Kups_PrinterView::initWidgets()
{
	bool	value(true);

	if (!initPrinterWidgets())
	{
		value = false;
	}
	if (value && !initClassWidgets())
	{
		value = false;
	}
	all_->setText(0, i18n("CUPS on %1:%2").arg(Kups_Infos::get()->getServer()).arg(Kups_Infos::get()->getPort()));

	return value;
}

bool Kups_PrinterView::initPrinterWidgets()
{
	KPrinterManager* prMgr = KPrinterManager::get();
	if (prMgr == 0)
	{
		return false;
	}

	allPrinters_->setAllChildrenDiscarded(true);

	int i;
	int nbPrinters = prMgr->getNbPrinters();
	for (i=0;i<nbPrinters;i++)
	{
		KRealPrinter* printer = prMgr->getPrinterAt(i);
		if (printer->isDiscarded()) continue;

		allPrinters_->addChild(printer, Kups_PrinterItem::PRINTER);
	}

	allPrinters_->deleteAllChildrenDiscarded();

	emit selectionChanged(currentItem());

	return true;	
}

bool Kups_PrinterView::initClassWidgets()
{
	KPrinterManager* prMgr = KPrinterManager::get();
	if (prMgr == 0)
	{
		return false;
	}
	
	allClasses_->setAllChildrenDiscarded(true);
	
	int i;
	int nbClasses = prMgr->getNbClasses();
	for (i=0;i<nbClasses;i++)
	{
		KClassPrinter* printer = prMgr->getClassAt(i);
		if (printer->isDiscarded()) continue;

		Kups_PrinterItem* child = allClasses_->addChild(printer,Kups_PrinterItem::CLASS);
		
		updatePrintersForClass(child);
	}
	
	allClasses_->deleteAllChildrenDiscarded();
	
	return true;
}

//----------------------------------------------------------------------
void Kups_PrinterView::resizeEvent(QResizeEvent * e )
{
	QListView::resizeEvent(e);
	
	if (allColumns_)
	{
	}
	else
	{
		setColumnWidthMode(0, Maximum);
		setColumnWidth(0, size().width());
	}
}

//----------------------------------------------------------------------
void Kups_PrinterView::updatePrintersForClass(Kups_PrinterItem* parent)
{
	KClassPrinter* classe = (KClassPrinter*)parent->getPrinter();
	
	parent->setAllChildrenDiscarded(true);
	
	int nbClasses = classe->getNbPrinters();
	for (int i=0;i<nbClasses;i++)
	{
		KPrinter* printer = classe->getPrinterAt(i);
		
		parent->addChild(printer, Kups_PrinterItem::PRINTER);
	}

	parent->deleteAllChildrenDiscarded();
}

KPrinter* Kups_PrinterView::getSelectedPrinter()
{
	Kups_PrinterItem* item = (Kups_PrinterItem*)currentItem();
	if (item == 0)
	{
		return 0;
	}
	return item->getPrinter();
}

#define	ID_POP_DEFAULT	0
#define	ID_POP_ENABLE	1
#define	ID_POP_DISABLE	2
#define	ID_POP_TEST	3
#define	ID_POP_ADD	4
#define	ID_POP_REMOVE	5
#define	ID_POP_CLASS	6
#define	ID_POP_CONFIG	7
#define	ID_POP_REFRESH	8
#define	ID_POP_DEFAULT2	9
#define	ID_POP_ADDTOCLASS	100
#define	ID_POP_ADDPRINTER	200
void Kups_PrinterView::slotRightOnItem(QListViewItem* it, const QPoint& point, int )
{
	Kups_PrinterItem* item = (Kups_PrinterItem*)it;
	KPrinter* printer = getSelectedPrinter();

	getView()->stopTimer();

	QPopupMenu menu(0), menu2(0);//this);

	if (printer && item)
	{
		menu.insertItem( SmallIcon((printer->isClass() || printer->isImplicit() ? "kups_class_default" : "kups_printer_default")), i18n("Set as local default"), ID_POP_DEFAULT);
		if (printer->isRemote())	// can't set default for remote printer/class
			menu.setItemEnabled(ID_POP_DEFAULT, false);
		menu.insertItem( SmallIcon("exec"), (getuid() == 0 ? i18n("Set as system default") : i18n("Set as personal default")), ID_POP_DEFAULT2);
		menu.insertSeparator();
		if (item->getPrinterType() == Kups_PrinterItem::CLASS)
		{
			if (printer->getState() == KPrinter::Stopped)
				menu.insertItem( SmallIcon("kups_enable"), i18n("Enable class"), ID_POP_ENABLE);
			else menu.insertItem( SmallIcon("stop"), i18n("Disable class"), ID_POP_DISABLE);
			menu.insertSeparator();
			if (!printer->isRemote())
			{
				int	nbPrinters = KPrinterManager::get()->getNbPrinters();
				for (int i=0;i<nbPrinters;i++)
				{
					KPrinter	*pr = KPrinterManager::get()->getPrinterAt(i);
					if (!pr || item->getChildWithPrinter(pr)) continue;
					Kups_PrinterItem	*pItem = allPrinters_->getChildWithPrinter(pr);
					if (pItem) menu2.insertItem( *pItem->pixmap(0), pr->getName(), ID_POP_ADDTOCLASS+i+1);
				}
				menu.insertItem( SmallIcon("kups_add"), i18n("Add printer..."), &menu2, ID_POP_ADDTOCLASS);
				if ((int)(menu2.count()) == 0) menu.setItemEnabled(ID_POP_ADDTOCLASS,false);
			}
			menu.insertItem( SmallIcon("trashcan_empty"), i18n("Remove class"), ID_POP_REMOVE);
		}
		else if (item->getPrinterType() == Kups_PrinterItem::PRINTER)
		{
			bool	implicit_ = printer->isImplicit();
			if (printer->getState() == KPrinter::Stopped)
				menu.insertItem( SmallIcon("kups_enable"), (implicit_ ? i18n("Enable class") : i18n("Enable printer")), ID_POP_ENABLE);
			else menu.insertItem( SmallIcon("stop"), (implicit_ ? i18n("Disable class") : i18n("Disable printer")), ID_POP_DISABLE);
			menu.insertItem( SmallIcon("fileprint"), (implicit_ ? i18n("Test class") : i18n("Test printer")), ID_POP_TEST);
			menu.insertSeparator();
			if (item->depth() == 2)
			{
				int	nbClasses = KPrinterManager::get()->getNbClasses();
				for (int i=0;i<nbClasses;i++)
				{
					KClassPrinter	*cl = KPrinterManager::get()->getClassAt(i);
					if (!cl || cl->hasPrinter(printer) || cl->isRemote()) continue;
					Kups_PrinterItem	*cItem = allClasses_->getChildWithPrinter(cl);
					if (cItem) menu2.insertItem( *cItem->pixmap(0), cl->getName(), ID_POP_ADDPRINTER+i+1);
				}
				menu.insertItem( SmallIcon("kups_class"), i18n("Add to class..."), &menu2, ID_POP_ADDPRINTER);
				if ((int)(menu2.count()) == 0) menu.setItemEnabled(ID_POP_ADDPRINTER,false);
				if (!printer->isImplicit()) menu.insertItem( SmallIcon("trashcan_empty"), i18n("Remove printer"), ID_POP_REMOVE);
			}
			else if (item->depth() == 3)
			{
				menu.insertItem( SmallIcon("trashcan_empty"), i18n("Remove from class"), ID_POP_CLASS);
				// disable item if last printer in class
				if (item->parent()->childCount() == 1u) menu.setItemEnabled(ID_POP_CLASS,false);
			}
			if (!implicit_)
			{
				menu.insertSeparator();
				menu.insertItem( SmallIcon("configure"), i18n("Configure printer"), ID_POP_CONFIG);
				if (printer->getModelInfo() == "Local Raw Printer")
					menu.setItemEnabled(ID_POP_CONFIG, false);
			}
		}
		menu.insertSeparator();
		menu.insertItem( SmallIcon("reload"), i18n("Refresh"), ID_POP_REFRESH);
	}
	else if (item && item->depth() == 1 && currentItem())
	{
		menu.insertItem( SmallIcon("wizard"), i18n("Add printer/class ..."), ID_POP_ADD);
		menu.insertSeparator();
		menu.insertItem( SmallIcon("reload"), i18n("Refresh"), ID_POP_REFRESH);
	}
	else	// do nothing
	{
		getView()->startTimer();
		return;
	}

	int ret = menu.exec(point);
	bool needRefresh(true);
#ifdef DEBUG
debug("menu value: %d", ret);
#endif

	bool retVal = true;
	switch (ret)
	{
	   case ID_POP_DEFAULT:	// set default
		retVal = printer->sendCommand(KPrinter::AsDefault);
		break;
	   case ID_POP_DEFAULT2:// set as personal default
		retVal = KInstanceManager::get()->setDefaultInstance(printer->getName(), i18n("(Default)"));
		if (!retVal) KPrinterManager::get()->setLastError(KInstanceManager::get()->getLastError());
		break;
	   case ID_POP_ENABLE:	// enable printer
		retVal = printer->sendCommand(KPrinter::Enable);
		break;
	   case ID_POP_DISABLE:	// disable printer
		retVal = printer->sendCommand(KPrinter::Disable);
		break;
	   case ID_POP_TEST:	// test printer
		retVal = Kups_TestPageGenerator::printPage(this, printer);
		if (!retVal)
		{
			retVal = true;
			needRefresh = false;
		}
		break;
	   case ID_POP_ADD:	// add printer
		{
		Kap_AddPrinterWizard dlg(this);
		retVal = dlg.exec();
		if (!retVal) needRefresh = false;
		break;
		}
	   case ID_POP_REMOVE:	// remove printer
		if (QMessageBox::warning(this,i18n("Remove"),i18n("Do you really want to remove %1 \"%2\" ?").arg(printer->isA("KClassPrinter") ? i18n("class") : i18n("printer")).arg(printer->getName()),QMessageBox::Yes|QMessageBox::Default,QMessageBox::No) == QMessageBox::Yes)
			retVal = printer->sendCommand(KPrinter::Remove);
		else needRefresh = false;
		break;
	   case ID_POP_CONFIG:	// configure printer
		retVal = Kups_PrinterConfigurator::configurePrinter(printer, this);
		if (!retVal && KPrinterManager::getLastError().isEmpty())	// action has been cancelled
		{
			retVal = true;
			needRefresh = false;
		}
		break;
	   case ID_POP_CLASS:	// remove from class
	   	{
		Kups_PrinterItem* itClasse = (Kups_PrinterItem*)item->parent();
		KPrinter* classe = itClasse->getPrinter();
		if (QMessageBox::warning(this,i18n("Remove"),i18n("Do you really want to remove \"%1\" from \"%2\" ?").arg(printer->getName()).arg(classe->getName()),QMessageBox::Yes|QMessageBox::Default,QMessageBox::No) == QMessageBox::Yes)
			retVal = printer->sendCommand(KPrinter::RemoveFromClass, classe);
		else needRefresh = false;
		break;
		}
	   case ID_POP_REFRESH:	// refresh (done automatically if retVal is true)
		retVal = true;
		break;
	   default:
	   	if (ret > 100 && ret < 200)
	   	{ // add printer to class (from class)
	   		int	pIndex = ret - ID_POP_ADDTOCLASS - 1;
	   		KPrinter	*pr = KPrinterManager::get()->getPrinterAt(pIndex);
	   		if (pr) retVal = pr->sendCommand(KPrinter::AddToClass, printer);
	   	}
	   	else if (ret > 200 && ret < 300)
	   	{ // add printer to class (from printer)
	   		int	cIndex = ret - ID_POP_ADDPRINTER - 1;
	   		KPrinter	*cl = KPrinterManager::get()->getClassAt(cIndex);
	   		if (cl) retVal = printer->sendCommand(KPrinter::AddToClass, cl);
	   	}
	   	else
	   	{
	   		retVal = true;
	   		needRefresh = false;
	   	}
	   	break;
	}
	
	// do not take into account cases where retVal is false and action is add a printer
	// because if there's an error, it has already been shown in the dialog and this avoids
	// to have an error message when the user clicks "Cancel"
	if (retVal == false && ret != ID_POP_ADD)
	{
		QString str = i18n("ERROR : %1").arg(KPrinterManager::getLastError());
		QMessageBox::critical(this,i18n("Error"),str,QMessageBox::Ok|QMessageBox::Default,0);
		KPrinterManager::setPassword("");
	}
	else if (needRefresh)
	{
		KPrinterManager::get()->refresh();
	}
	
	getView()->startTimer();
	return;
}
